package cz.muni.fi.pb138.dtdToXsd;

import java.io.*;
import java.util.List;
import java.util.logging.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

/**
 * PB138 Project: DTD to XML Schema
 *
 * @author Veronika Lovrantova 373905
 * @author Michal Merta 374015
 * @author Pavel Demuth 374569
 * @author Roman Jakubco 374241
 *
 */
public class App {

    public static void main(String[] args) {
        Handler handler;
        try {
            handler = new FileHandler("logFile.xml");
            handler.setFormatter(new XMLFormatter());
            Logger.getLogger("").addHandler(handler);
        } catch (IOException ex) {
            Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
        } catch (SecurityException ex) {
            Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
        }

        String path = null;
        Document doc = null;
        File dtdDoc = null;
        File dtdDocXml = null;
        File vystup = null;
        File temp = null;
        File temp2 = null;
        FileInputStream newFile = null;
        FileInputStream newFile2 = null;
        FileInputStream dtd = null;
        FileInputStream dtdXml = null;
        String workingDir = System.getProperty("user.dir").concat(File.separator);
        FileOutputStream fos = null;

        if (args.length == 0) {
            System.out.println("Program: DTD to XSD schema transformation");
            System.out.println("Version: 1.0");
            System.out.println("Authors: Pavel Demuth, Roman Jakubco, Veronika Lovrantova, Michal Merta");
            System.out.println("Run data: Program needs to be run with one parameter, the full path to XML file or just name of XML file in actual directory.\n");
            return;
        }
        if (args.length != 1) {
            System.err.println("Error: More than one paramater \n");
            return;
        }
        if(!args[0].toLowerCase().contains(".xml")){
            System.err.println("Error: Selected file is not a XML file \n");
            return;
        }
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            doc = builder.parse(args[0]);

        } catch (ParserConfigurationException ex) {
            System.err.println("Parser Configuration error:" + ex.getMessage() + "\n");
            java.util.logging.Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
            return;
        } catch (SAXException ex) {
            System.err.println("Error in SAX:" + ex.getMessage() + "\n");
            java.util.logging.Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
            return;
        } catch (IOException ex) {
            System.err.println("Error :" + ex.getMessage() + "\n");
            java.util.logging.Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
            return;
        }
        
        if (args[0].contains(File.separator)) {
            path = args[0].substring(0, (args[0].lastIndexOf(File.separator) + 1));

        }

        try {
            DTDtoXSDConvertorImpl convertorImpl = new DTDtoXSDConvertorImpl();
            DTDCutterImpl cutter = new DTDCutterImpl();
            String name = null;
            try {
                name = doc.getDoctype().getSystemId();
                
            } catch (NullPointerException ex) {
                System.err.println("Error: There is not DTD reference in selected XML file \n");                
                return;
            }
            
            boolean testPath = false;
            if (path != null) {
                testPath = true;
            }

            //ci je interne
            if (name == null) {
                dtdDocXml = new File(args[0]);
            } else {                
                if (testPath) {
                    dtdDoc = new File(path.concat(name));
                } else {
                    dtdDoc = new File(name);
                }
                dtdDocXml = new File(args[0]);
                dtd = new FileInputStream(dtdDoc);

                temp = cutter.prepareFile(dtd);
                newFile = new FileInputStream(temp);
                cutter.parseDTD(newFile);

            }

            String docTypeName = doc.getDoctype().getName();


            dtdXml = new FileInputStream(dtdDocXml);
            temp2 = cutter.prepareInternFile(dtdXml);
            newFile2 = new FileInputStream(temp2);
            cutter.parseDTD(newFile2);


            List<ElementDTD> elements = cutter.getElements();
            List<Entity> entities = cutter.getEntities();
            List<Attribute> attributes = cutter.getAttributes();
            List<String> comments = cutter.getComments();


            Document schema = convertorImpl.convertDTD(elements, attributes, entities, comments);
            PrinterImpl printer = new PrinterImpl(doc);

            //na odlisene od ostatnych xsd preto +New
            if (testPath) {
                vystup = new File(path + docTypeName + "New.xsd");
            } else {
                vystup = new File(docTypeName + "New.xsd");
            }

            fos = new FileOutputStream(vystup);
            printer.printDocument(schema, fos);
            System.out.println(elements);
            System.out.println(attributes);

            Node root = doc.getDocumentElement();
            Attr attribute2 = doc.createAttribute("xmlns");
            Attr attribute3 = doc.createAttribute("xmlns:xsi");
            Attr attribute1 = doc.createAttribute("xsi:noNamespaceSchemaLocation");

            attribute1.setValue(docTypeName + "New.xsd");
            attribute2.setValue("");
            attribute3.setValue("http://www.w3.org/2001/XMLSchema-instance");

            NamedNodeMap schemaAttributes = root.getAttributes();
            schemaAttributes.setNamedItem(attribute2);
            schemaAttributes.setNamedItem(attribute3);
            schemaAttributes.setNamedItem(attribute1);

            //na odlisene od ostatnych xsd preto +vysledok
            System.out.println("\n");
            System.out.println("New schema created: " + docTypeName + "New.xsd \n");
            printer.printDocument(schema, System.out);

            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            StreamResult result = new StreamResult(args[0]);
            DOMSource source = new DOMSource(doc);
            transformer.transform(source, result);

        } catch (TransformerException ex) {
            System.err.println("Error:" + ex.getLocalizedMessage() + "\n");
            java.util.logging.Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
        } catch (XSDConvertException ex) {
            System.err.println("Error:" + ex.getLocalizedMessage() + "\n");
            java.util.logging.Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
        } catch (FileNotFoundException ex) {
            System.err.println("Error:" + ex.getLocalizedMessage() + "\n");
            java.util.logging.Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
        } catch (NullPointerException ex) {
            System.err.println("Error:" + ex.getLocalizedMessage() + "\n");
            java.util.logging.Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ParserConfigurationException ex) {
            System.err.println("Error:" + ex.getLocalizedMessage() + "\n");
            java.util.logging.Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                if ((dtd != null)) {
                    dtd.close();
                }
                if ((newFile != null)) {
                    newFile.close();
                }
                if ((newFile2 != null)) {
                    newFile2.close();
                }
                if (fos != null) {
                    fos.close();
                }
                if (dtdXml != null) {
                    dtdXml.close();
                }
                if (temp != null) {
                    temp.delete();
                }
                if (temp2 != null) {
                    temp2.delete();
                }
            } catch (IOException ex) {
                System.err.println("Error:" + ex.getLocalizedMessage() + "\n");
                java.util.logging.Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
}
