/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.pb138.dtdToXsd;

import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 *
 * @author Michal
 */
public interface DTDtoXSDConvertor {
    
     /**
     * method transforms given ElementDTDs into XSD representation and attaches them into document.
     * It's necessary to use this method before using convertAttributes.
     *
     *
     * @param inputElements - sorted list of ElementDTD objects, their order is defined in DTDCutter
     * @throws XSDConvertException when some error occurs during conversion
     */
     public void convertElements(List<ElementDTD> inputElements) throws XSDConvertException;

    /**
    * method converts given ElementDTD into it's XSD representation and adds it into document.
    * 
    * @param inputElement 
    * @param parentElement - this element is set as parent of converted schema. If it's null, root element is used
    * @throws XSDConvertException 
    */
    public void convertElement(ElementDTD inputElement,Element parentElement) throws XSDConvertException;

    /**
     * converts each Attribute into it's XSD representation and attaches it into document
     *
     * @param inputAttributes - each Attribute contains information about one DTD attribute
     * @throws XSDConvertException when some error occurs during conversion
     */
    public void convertAttributes(List<Attribute> inputAttributes) throws XSDConvertException;

    /**
     * method parses input Attribute and converts it into XML Schema.
     * All parts of result schema
     * are attached into document.
     * 
     * @param inputAttribute 
     * 
     * @throws XSDConvertException when some error occurs during conversion
     */
    public void convertAttribute(Attribute inputAttribute) throws XSDConvertException;
    
    /**
     * add each comment into documentation of XSD document
     * 
     * @param comments - list of comments
     */

    public void convertComments(List<String> comments);

    
    /**
     * creates Document containing XSD schema from given elements, attributes and comments 
     * 
     * @param elements list of ElemntDTD objects
     * @param attribute list of Attribute objects
     * @param entities unused - list of Entity objects
     * @param comments list of String
     * @return Document containing XSD schema
     * @throws XSDConvertException 
     */

    public Document convertDTD(List<ElementDTD> elements, List<Attribute> attribute, 
        List<Entity> entities, List<String> comments) throws XSDConvertException;
    
}
